/* vim: set expandtab ts=8 sw=4 encoding=utf-8: */

/*  $Id$
 *
 *  Copyright © 2004 Jasper Huijsmans <jasper@xfce.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <signal.h>
#include <string.h>
#include <errno.h>

#include <libxfce4util/libxfce4util.h>
#include <libxfcegui4/libxfcegui4.h>

#ifndef _
#define _(x) x
#define N_(x) x
#endif

/* globally registered helper apps */
struct xfce_app
{
    char *name;
    char *icon;
    char *cmd;
    pid_t pid;
};

struct xfce_app apps [] =
{
    {N_("Desktop Manager"), "xfce4-backdrop", "xfdesktop", -1},
    {N_("Panel"), "xfce4-panel", "xfce4-panel", -1},
    {N_("Taskbar"), "xfce4-taskbar", "xftaskbar4", -1},
    {N_("Icon Box"), "xfce4-iconbox", "xfce4-iconbox", -1}
};


/* prototypes */
static void find_pid (struct xfce_app *app);

static void run_dialog (void);


/* main */
int
main (int argc, char **argv)
{
    int i;

    if (argc > 1 && *argv[1] == '-')
    {
	if (!strcmp (argv [1], "-v") || !strcmp (argv [1], "--version") ||
	    !strcmp (argv [1], "--version") || !strcmp (argv[1], "--help"))
	{
	    g_print (_("Xfce Start/Stop %s\n"), VERSION);
	}
	
	exit (0);
    }
    
    gtk_init (&argc, &argv);

    xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");

    if ((i = gdk_display_get_n_screens (gdk_display_get_default ())) > 1)
    {
        char title[256];

        g_snprintf (title, 256, _("%d screens detected"), i);
        
        xfce_message_dialog (NULL,
                _("Sorry"), GTK_STOCK_DIALOG_ERROR, title,
                _("Unfortunately this program can not properly handle "
                  "a system with more than one screen."),
                GTK_STOCK_CLOSE, GTK_RESPONSE_OK, NULL);

        exit (1);
    }
    
    for (i=0; i < G_N_ELEMENTS (apps); ++i)
    {
        find_pid (&apps[i]);
    }

    run_dialog ();

    exit (0);
}

static void 
find_pid (struct xfce_app *app)
{
    char *out = NULL;
    pid_t pid;
    char *argv[] = { "pidof", app->cmd, NULL };

    if (g_spawn_sync (NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, 
                      &out, NULL, NULL, NULL) && out != NULL)
    {
        if (sscanf (out, "%d", &pid) > 0)
            app->pid = pid;
    }
}

static void
toggle_app (GtkToggleButton *tb, struct xfce_app *app)
{
    if (gtk_toggle_button_get_active (tb))
    {
        GPid pid;
        char *argv[] = {app->cmd, NULL};

        if (app->pid == -1)
        {
            if (g_spawn_async_with_pipes (NULL, argv, NULL, 
                        G_SPAWN_SEARCH_PATH, NULL, NULL, 
                        &pid, NULL, NULL, NULL, NULL))
            {
                app->pid = (pid_t)pid;
            }
        }
    }
    else if (app->pid > 1)
    {
        if (kill (app->pid, SIGTERM) == 0)
        {
            app->pid = -1;
        }
        else
        {
            perror (strerror(errno));
            gtk_toggle_button_set_active (tb, TRUE);
        }
    }
}

static void
run_dialog (void)
{
    GtkWidget *dlg, *header, *vbox, *hbox, *check, *im, *label;
    GdkPixbuf *pb;
    GtkSizeGroup *sg;
    int i;

    dlg = gtk_dialog_new_with_buttons (_("Start/Stop"), NULL,
            GTK_DIALOG_NO_SEPARATOR, GTK_STOCK_CLOSE, GTK_RESPONSE_OK, NULL);
    
    pb = xfce_themed_icon_load ("xfce-utils", 48);
    header = xfce_create_header (pb, _("Start/Stop Xfce Components"));
    gtk_widget_show (header);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), header, 
                        FALSE, FALSE, 0);
    if (pb)
        g_object_unref (pb);

    vbox = gtk_vbox_new (FALSE, 10);
    gtk_widget_show (vbox);
    gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), vbox, 
                        FALSE, FALSE, 6);
    
    sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
    
    for (i=0; i < G_N_ELEMENTS (apps); ++i)
    {
        char *path;

        path = g_find_program_in_path (apps[i].cmd);

        if (path)
            g_free (path);
        else
            continue;
        
        hbox = gtk_hbox_new (FALSE, 12);
        gtk_widget_show (hbox);
        gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

        check = gtk_check_button_new ();
        gtk_widget_show (check);
        gtk_box_pack_start (GTK_BOX (hbox), check, FALSE, FALSE, 0);

        if (apps[i].pid > 1)
            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);

        g_signal_connect (check, "toggled", G_CALLBACK (toggle_app), &apps[i]);

        pb = xfce_themed_icon_load (apps[i].icon, 32);
        if (pb)
        {
            im = gtk_image_new_from_pixbuf (pb);
            g_object_unref (pb);
        }
        else
        {
            im = gtk_alignment_new (0, 0, 0, 0);
        }

        gtk_widget_show (im);
        gtk_size_group_add_widget (sg, im);
        gtk_box_pack_start (GTK_BOX (hbox), im, FALSE, FALSE, 0);

        label = gtk_label_new (_(apps[i].name));
        gtk_widget_show (label);
        gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
        gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
    }

    gtk_dialog_run (GTK_DIALOG (dlg));

    gtk_widget_destroy (dlg);
}

